@@ -3,82 +3,67 @@
 # *************************************************
 
 
-# ****** Internal functions used by drawing code ******
-plot_to_gtable <- function(plot){
-  if (methods::is(plot, "function") || methods::is(plot, "recordedplot")){
-    if (!requireNamespace("gridGraphics", quietly = TRUE)){
-      warning("Package `gridGraphics` is required to handle base-R plots. Substituting empty plot.", call. = FALSE)
-      u <- grid::unit(1, "null")
-      gt <- gtable::gtable_col(NULL, list(grid::nullGrob()), u, u)
-      # fix gtable clip setting
-      gt$layout$clip <- "inherit"
-      gt
-    }
-    else {
-      tree <- grid::grid.grabExpr(gridGraphics::grid.echo(plot))
-      u <- grid::unit(1, "null")
-      gt <- gtable::gtable_col(NULL, list(tree), u, u)
-      # fix gtable clip setting
-      gt$layout$clip <- "inherit"
-      gt
-    }
-  }
-  else if (methods::is(plot, "ggplot")){
-    # ggplotGrob must open a device and when a multiple page capable device (e.g. PDF) is open this will save a blank page
-    # in order to avoid saving this blank page to the final target device a NULL device is opened and closed here to *absorb* the blank plot
 
-    grDevices::pdf(NULL)
-    plot <- ggplot2::ggplotGrob(plot)
-    grDevices::dev.off()
-    plot
-  }
-  else if (methods::is(plot, "gtable")){
-    plot
-  }
-  else{
-    stop(
-      'Argument needs to be of class "ggplot", "gtable", "recordedplot", ',
-      'or a function that plots to an R graphics device when called, ',
-      'but is a ', class(plot))
-  }
-}
-
-
-#' Draw a line.
+#' Draw a line from connected points
 #'
-#' This is a convenience function. It's just a thin wrapper around \code{geom_line}.
+#' Provide a sequence of x values and accompanying y values to draw a line on a plot.
+#'
+#' This is a convenience function, providing a wrapper around ggplot2's \code{geom_path}.
 #'
 #' @param x Vector of x coordinates.
 #' @param y Vector of y coordinates.
-#' @param ... Style parameters, such as \code{colour}, \code{alpha}, \code{size}, etc.
+#' @param ... geom_path parameters such as \code{colour}, \code{alpha}, \code{size}, etc.
+#' @seealso \code{\link{geom_path}}, \code{\link{ggdraw}}
+#' @examples
+#' ggdraw() + draw_line(x = c(0.2, 0.7, 0.7, 0.3),
+#'                      y = c(0.1, 0.3, 0.9, 0.8),
+#'                      color = "blue", size = 2)
 #' @export
 draw_line <- function(x, y, ...){
   geom_path(data = data.frame(x, y),
             aes(x = x, y = y),
+            inherit.aes = FALSE,
             ...)
 }
 
-#' Draw text.
+#' Draw multiple text-strings in one go.
 #'
 #' This is a convenience function to plot multiple pieces of text at the same time. It cannot
 #' handle mathematical expressions, though. For those, use \code{draw_label}.
 #'
-#' Note that font sizes get scaled by a factor of 2.85, so sizes given here agree with font sizes used in
+#' Note that font sizes are scaled by a factor of 2.85, so sizes agree with those of
 #' the theme. This is different from \code{geom_text} in ggplot2.
 #'
 #' By default, the x and y coordinates specify the center of the text box. Set \code{hjust = 0, vjust = 0} to specify
 #' the lower left corner, and other values of \code{hjust} and \code{vjust} for any other relative location you want to
 #' specify.
-#' @param text Character or expression vector specifying the text to be written.
+#'
+#' For a full list of ... options, see  \code{\link{geom_label}}.
+#'
+#' @param text A vector of Character (not expressions) specifying the string(s) to be written.
 #' @param x Vector of x coordinates.
 #' @param y Vector of y coordinates.
+#' @param hjust (default = 0.5)
+#' @param vjust (default = 0.5)
 #' @param size Font size of the text to be drawn.
 #' @param ... Style parameters, such as \code{colour}, \code{alpha}, \code{angle}, \code{size}, etc.
+#' @seealso \code{\link{draw_label}}
+#' @examples
+#' # Draw onto a 1*1 drawing surface
+#' ggdraw() + draw_text("Hello World!", x = 0.5, y = 0.5)
+#' #
+#' # Adorn a plot from the Anscombe data set of "identical" data.
+#' p = qplot(x = x1, y = y1, geom = c("smooth", "point"), data = anscombe)
+#' threeStrings = c("Hello World!", "to be or not to be", "over and out")
+#' p + draw_text(threeStrings, x = 8:10, y = 5:7, hjust = 0)
 #' @export
-draw_text <- function(text, x = 0.5, y = 0.5, size = 14, ...){
+draw_text <- function(text, x = 0.5, y = 0.5, size = 14, hjust = 0.5, vjust = 0.5, ...){
   geom_text(data = data.frame(text, x, y),
-            aes(label = text, x = x, y = y),
-            size = size / .pt, # scale font size to match size in theme definition
+            aes(x = x, y = y, label = text),
+            size = (size / .pt), # scale font size to match size in theme definition
+            inherit.aes = FALSE,
+			hjust = hjust,
+			vjust = vjust,
             ...)
 }
 
@@ -93,10 +78,10 @@ draw_text <- function(text, x = 0.5, y = 0.5, size = 14, ...){
 #' the lower left corner, and other values of \code{hjust} and \code{vjust} for any other relative location you want to
 #' specify.
 #' @param label String or plotmath expression to be drawn.
-#' @param x The x location of the label.
-#' @param y The y location of the label.
-#' @param hjust Horizontal justification
-#' @param vjust Vertical justification
+#' @param x The x location (origin) of the label.
+#' @param y The y location (origin) of the label.
+#' @param hjust Horizontal justification. Default = 0.5 (centered on x). 0 = flush-left at x, 1 = flush-right.
+#' @param vjust Vertical justification. Default = 0.5 (centered on y). 0 = baseline at y, 1 = ascender at y.
 #' @param fontfamily The font family
 #' @param fontface The font face ("plain", "bold", etc.)
 #' @param colour Text color
@@ -104,15 +89,31 @@ draw_text <- function(text, x = 0.5, y = 0.5, size = 14, ...){
 #' @param angle Angle at which text is drawn
 #' @param lineheight Line height of text
 #' @param alpha The alpha value of the text
+#' @seealso \code{\link{ggdraw}}
 #' @examples
-#' p <- ggplot(mtcars, aes(mpg, disp)) + geom_line(colour = "blue") + background_grid(minor='none')
-#' c <- cor.test(mtcars$mpg, mtcars$disp, method='sp')
+#' # setup plot and a label (regression description)
+#' p <- ggplot(mtcars, aes(mpg, disp)) + geom_line(color = "blue") + background_grid(minor = 'none')
+#' c <- cor.test(mtcars$mpg, mtcars$disp, method = 'sp')
 #' label <- substitute(paste("Spearman ", rho, " = ", estimate, ", P = ", pvalue),
 #'                     list(estimate = signif(c$estimate, 2), pvalue = signif(c$p.value, 2)))
-#' # adding label via ggdraw, in the ggdraw coordinates
-#' ggdraw(p) + draw_label(label, .7, .9)
-#' # adding label directly to plot, in the data coordinates
-#' p + draw_label(label, 20, 400, hjust = 0, vjust = 0)
+#'
+#' # Add label to plot, centered on {x,y} (in data coordinates)
+#' p + draw_label(label, x = 20, y = 400)
+#' # Add label to plot in data coordinates, flush-left at x, baseline at y.
+#' p + draw_label(label, x = 20, y = 400, hjust = 0, vjust = 0)
+#'
+#' # Add label to plot. Data coordinates, drawing rightward
+#' # from x, with ascenders of text touching y.
+#' p + draw_label(label, x = 20, y = 400, hjust = 0, vjust = 1)
+#'
+#' # Add labels via ggdraw. Uses ggdraw coordinates.
+#' # ggdraw coordinates default to xlim = c(0, 1), ylim = c(0, 1).
+#' ggdraw(p) + draw_label("centered on 70% of x, 90% of y height", x = 0.7, y = 0.9)
+#' labstr = "bottom left at {0%, 0%} of the SHEET, not the plot!"
+#' p = ggdraw(p) + draw_label(labstr, x = 0, y = 0, hjust = 0, vjust = 0)
+#' p = p + draw_label("top right at {1,1}", x = 1, y = 1, hjust = 1, vjust = 1)
+#' p = p + draw_label("bottom left at {.4,.4}", x = 0.4, y = 0.4, hjust = 0, vjust = 0)
+#' p + draw_label("centered on at {.5,.5}", x = 0.5, y = 0.5, hjust = 0.5, vjust = 0.5)
 #' @export
 draw_label <- function(label, x = 0.5, y = 0.5, hjust = 0.5, vjust = 0.5,
                     fontfamily = "", fontface = "plain", colour = "black", size = 14,
@@ -148,7 +149,7 @@ draw_label <- function(label, x = 0.5, y = 0.5, hjust = 0.5, vjust = 0.5,
 #' @param colour (optional) Color of the plot labels. If not provided, is taken from the current theme.
 #' @param ... Other arguments to be handed to \code{draw_text}.
 #' @export
-draw_plot_label <- function(label, x=0, y=1, hjust = -0.5, vjust = 1.5, size = 16, fontface = 'bold',
+draw_plot_label <- function(label, x = 0, y = 1, hjust = -0.5, vjust = 1.5, size = 16, fontface = 'bold',
                             family = NULL, colour = NULL, ...){
   if (is.null(family)) {
     family <- theme_get()$text$family
@@ -165,14 +166,15 @@ draw_plot_label <- function(label, x=0, y=1, hjust = -0.5, vjust = 1.5, size = 1
 
 #' Add a label to a figure
 #'
-#' This function is similar to \code{draw_plot_label}, just with slightly different arguments and defaults. The main purpose of this
-#' function is to add labels specifying extra information about the figure, such as "Figure 1", which is sometimes useful.
+#' The main purpose of this function is to add labels specifying extra information about
+#' the figure, such as "Figure 1", or "A" - often useful in cowplots with more than
+#' one pane. The function is similar to \code{draw_plot_label}.
 #' @param label Label to be drawn
 #' @param position Position of the label, can be one of "top.left", "top", "top.right", "bottom.left", "bottom", "bottom.right". Default is "top.left"
 #' @param size (optional) Size of the label to be drawn. Default is the text size of the current theme
 #' @param fontface (optional) Font face of the label to be drawn. Default is the font face of the current theme
 #' @param ... other arguments passed to \code{draw_plot_label}
-#'
+#' @seealso \code{\link{draw_plot_label}}
 #' @examples
 #'
 #' p1 <- qplot(1:10, 1:10)
@@ -223,22 +225,87 @@ draw_figure_label <- function(label, position = c("top.left", "top", "top.right"
   )
 }
 
+#' Draw an image
+#'
+#' Places an image somewhere onto the drawing canvas. By default, coordinates run from
+#' 0 to 1, and the point (0, 0) is in the lower left corner of the canvas.
+#' @param image The image to place. Can be a file path, a URL, or a raw vector with image data,
+#'  as in [magick::image_read()]. Can also be an image previously created by [magick::image_read()] and
+#'  related functions.
+#' @param x The x location of the lower left corner of the image.
+#' @param y The y location of the lower left corner of the image.
+#' @param width Width of the image.
+#' @param height Height of the image.
+#' @param scale Scales the image relative to the rectangle defined by `x`, `y`, `width`, `height`. A setting
+#'   of `scale = 1` indicates no scaling.
+#' @param clip Set to "on" to clip the image relative to the box into which it is draw (useful for `scale > 1`).
+#'   Note that clipping doesn't always work as expected, due to limitations of the grid graphics system.
+#' @param interpolate A logical value indicating whether to linearly interpolate the image
+#'  (the alternative is to use nearest-neighbour interpolation, which gives a more blocky result).
+#' @examples
+#' # Use image as plot background
+#' p <- ggplot(iris, aes(x = Sepal.Length, fill = Species)) + geom_density(alpha = 0.7)
+#' ggdraw() +
+#'   draw_image("http://jeroen.github.io/images/tiger.svg") +
+#'   draw_plot(p + theme(legend.box.background = element_rect(color = "white")))
+#'
+#' # Manipulate images and draw in plot coordinates
+#' img <- magick::image_read("http://jeroen.github.io/images/tiger.svg")
+#' img <- magick::image_transparent(img, color = "white")
+#' img2 <- magick::image_negate(img)
+#' ggplot(data.frame(x = 1:3, y = 1:3), aes(x, y)) +
+#'   geom_point(size = 3) +
+#'   geom_abline(slope = 1, intercept = 0, linetype = 2, color = "blue") +
+#'   draw_image(img , x = 1, y = 1, scale = .9) +
+#'   draw_image(img2, x = 2, y = 2, scale = .9)
+#'
+#' # Make grid with plot and image
+#' p <- ggplot(iris, aes(x = Sepal.Length, fill = Species)) +
+#'   geom_density(alpha = 0.7)
+#' p2 <- ggdraw() + draw_image("http://jeroen.github.io/images/tiger.svg", scale = 0.9)
+#' plot_grid(p, p2, labels = "AUTO")
+#' @export
+draw_image <- function(image, x = 0, y = 0, width = 1, height = 1, scale = 1, clip = "inherit", interpolate = TRUE) {
+  if (!requireNamespace("magick", quietly = TRUE)){
+    warning("Package `magick` is required to draw images. Image not drawn.", call. = FALSE)
+    draw_grob(grid::nullGrob(), x, y, width, height)
+  }
+  else {
+    # if we're given an image, we just use it
+    if (methods::is(image, "magick-image")) {
+      image_data <- image
+    }
+    # otherwise we read it in with image_read()
+    else {
+      image_data <- magick::image_read(image)
+    }
+    g <- grid::rasterGrob(image_data, interpolate = interpolate)
+    draw_grob(g, x, y, width, height, scale, clip)
+  }
+}
+
 #' Draw a (sub)plot.
 #'
 #' Places a plot somewhere onto the drawing canvas. By default, coordinates run from
 #' 0 to 1, and the point (0, 0) is in the lower left corner of the canvas.
-#' @param plot The plot to place. Can be a ggplot2 plot, an arbitrary gtable,
-#'   or a recorded base-R plot, as in [plot_grid()].
+#' @param plot The plot to place. Can be a ggplot2 plot, an arbitrary grob or gtable,
+#'   or a recorded base-R plot, as in [plot_to_gtable()].
 #' @param x The x location of the lower left corner of the plot.
 #' @param y The y location of the lower left corner of the plot.
 #' @param width Width of the plot.
 #' @param height Height of the plot.
 #' @param scale Scales the grob relative to the rectangle defined by `x`, `y`, `width`, `height`. A setting
 #'   of `scale = 1` indicates no scaling.
+#' @examples
+#' # make a plot
+#' p <- qplot(1:10, 1:10)
+#' # draw into the top-right corner of a larger plot area
+#' ggdraw() + draw_plot(p, .6, .6, .4, .4)
 #' @export
-draw_plot <- function(plot, x = 0, y = 0, width = 1, height = 1, scale = 1){
-  g <- plot_to_gtable(plot) # convert to gtable if necessary
-  draw_grob(g, x, y, width, height, scale)
+draw_plot <- function(plot, x = 0, y = 0, width = 1, height = 1, scale = 1) {
+  if (!methods::is(plot, "grob"))
+    plot <- plot_to_gtable(plot) # convert to gtable if necessary
+  draw_grob(plot, x, y, width, height, scale)
 }
 
 #' Draw a grob.
@@ -254,8 +321,14 @@ draw_plot <- function(plot, x = 0, y = 0, width = 1, height = 1, scale = 1){
 #'   of `scale = 1` indicates no scaling.
 #' @param clip Set to "on" to clip the grob or "inherit" to not clip. Note that clipping doesn't always work as
 #'   expected, due to limitations of the grid graphics system.
+#' @examples
+#' # A grid grob (here a blue circle)
+#' library(grid)
+#' g <- circleGrob(gp = gpar(fill = "blue"))
+#' # place into the middle of the plotting area, at a scale of 50%
+#' ggdraw() + draw_grob(g, scale = 0.5)
 #' @export
-draw_grob <- function(grob, x = 0, y = 0, width = 1, height = 1, scale = 1, clip = "inherit"){
+draw_grob <- function(grob, x = 0, y = 0, width = 1, height = 1, scale = 1, clip = "inherit") {
   layer(
     data = data.frame(x = NA),
     stat = StatIdentity,
@@ -318,8 +391,10 @@ annotation_id <- local({
 
 
 #' Set up a drawing layer on top of a ggplot
-#' @param plot The plot to use as a starting point. Can be a ggplot2 plot, an arbitrary gtable,
-#'   or a recorded base-R plot, as in [plot_grid()].
+#'
+#' Set up a drawing layer on top of a ggplot.
+#' @param plot The plot to use as a starting point. Can be a ggplot2 plot, an arbitrary
+#'   grob or gtable, or a recorded base-R plot, as in [plot_to_gtable()].
 #' @param xlim The x-axis limits for the drawing layer.
 #' @param ylim The y-axis limits for the drawing layer.
 #' @examples
@@ -327,13 +402,12 @@ annotation_id <- local({
 #' ggdraw(p) + draw_label("Draft", colour = "grey", size = 120, angle = 45)
 #' @export
 ggdraw <- function(plot = NULL, xlim = c(0, 1), ylim = c(0, 1)) {
-
-  d <- data.frame(x=0:1, y=0:1) # dummy data
-  p <- ggplot(d, aes_string(x="x", y="y")) + # empty plot
+  d <- data.frame(x = 0:1, y = 0:1) # dummy data
+  p <- ggplot(d, aes_string(x = "x", y = "y")) + # empty plot
     scale_x_continuous(limits = xlim, expand = c(0, 0)) +
     scale_y_continuous(limits = ylim, expand = c(0, 0)) +
     theme_nothing() + # with empty theme
-    labs(x=NULL, y=NULL) # and absolutely no axes
+    labs(x = NULL, y = NULL) # and absolutely no axes
 
   if (!is.null(plot)){
     p <- p + draw_plot(plot)
