@@ -133,3 +133,26 @@ ggdraw() +
               theme(legend.justification = "top"), 0.5, 0.52, 0.5, 0.4) +
   draw_plot_label(c("A", "B"), c(0, 0.5), c(1, 0.92), size = 15)
 
+## ----message=FALSE-------------------------------------------------------
+p <- ggplot(iris, aes(x=Sepal.Length, fill=Species)) + geom_density(alpha = 0.7)
+ggdraw() +
+  draw_image("http://jeroen.github.io/images/tiger.svg") +
+  draw_plot(p)
+
+## ----message=FALSE, results="hide"---------------------------------------
+img <- magick::image_read("http://jeroen.github.io/images/tiger.svg")
+img <- magick::image_transparent(img, color = "white")
+img2 <- magick::image_negate(img)
+
+## ----message=FALSE-------------------------------------------------------
+ggplot(data.frame(x=1:3, y=1:3), aes(x, y)) +
+  geom_point(size = 3) +
+  geom_abline(slope = 1, intercept = 0, linetype = 2, color = "blue") +
+  draw_image(img, x=1, y=1, scale = .9) +
+  draw_image(img2, x=2, y=2, scale = .9)
+
+## ----message=FALSE, fig.width=7, fig.height=2.5--------------------------
+p <- ggplot(iris, aes(x = Sepal.Length, fill = Species)) + geom_density(alpha = 0.7)
+p2 <- ggdraw() + draw_image("http://jeroen.github.io/images/tiger.svg", scale = 0.9)
+plot_grid(p, p2, labels = "AUTO")
+
